﻿using DryIoc.ImTools;
using Google.Protobuf.WellKnownTypes;
using MySql.Data.MySqlClient;
using Newtonsoft.Json;
using StudentManager.Data;
using System.Diagnostics;
using System;
using Enum = System.Enum;
using System.Security.Cryptography;
using StudentManager.Interface;

namespace StudentManager.Common
{
	public static class SQLMapping
	{
		public static void Mapping<T>(MySqlDataReader reader)
		{
			typeof(T).GetProperties().ForEach(x => { Debug.WriteLine(x.Name.ToLower()); });
		}

		public static void Mapping<T>(T data, MySqlDataReader reader)
		{
			if (typeof(T) == typeof(StudentData))
			{
				(data as StudentData).UID = (long)reader[0];
				(data as StudentData).Name = reader[1].ToString();
				(data as StudentData).Gender = reader[2].ToString();
				(data as StudentData).Contact = reader[3].ToString();
				(data as StudentData).Address = reader[4].ToString();
				(data as StudentData).Grade = (byte)reader[5];
				(data as StudentData).TotalsQuestions = (UInt16)reader[6];
				(data as StudentData).CorrectionCount = (UInt16)reader[7];
				(data as StudentData).CurronHomeWorkIndex = (uint)reader[8];
				(data as StudentData).Password = reader[9].ToString();
				(data as StudentData).Birthdate = (DateTime)reader[10];
				(data as StudentData).HWQTotalCount = (uint)reader[11];
				(data as StudentData).HWQTotalErrorCount = (uint)reader[12];
			}
			else if (typeof(T) == typeof(QuestionData))
			{
				DifficultyLevel dLevel = DifficultyLevel.easy;
				QStatus qStatus = QStatus.published;
				QType qType = QType.填空题;
				(data as QuestionData).Id = (int)reader[0];
				Enum.TryParse(reader[1].ToString(), true, out qType);
				(data as QuestionData).Type = qType;
				(data as QuestionData).Stem = reader[2].ToString();
				(data as QuestionData).Answer = reader[3].ToString();
				Enum.TryParse(reader[4].ToString(), true, out dLevel);
				(data as QuestionData).DifficultyLevel = dLevel;
				(data as QuestionData).Category = reader[5].ToString();
				(data as QuestionData).Tags = reader[6].ToString();
				(data as QuestionData).Source = reader[7].ToString();
				(data as QuestionData).Lesson = (uint)reader[8];
				Enum.TryParse(reader[9].ToString(), true, out qStatus);
				(data as QuestionData).Status = qStatus;
			}
			else if (typeof(T) == typeof(CursonQuestionsData))
			{

				(data as CursonQuestionsData).UID = (long)reader[0];
				(data as CursonQuestionsData).ProblemIDS = string.IsNullOrEmpty(reader[1].ToString()) ? Array.Empty<int>() : JsonConvert.DeserializeObject<int[]>(reader[1].ToString());
				(data as CursonQuestionsData).CorrectIDS = string.IsNullOrEmpty(reader[2].ToString()) ? Array.Empty<int>() : JsonConvert.DeserializeObject<int[]>(reader[2].ToString());
				(data as CursonQuestionsData).Lesson = (uint)reader[3];
				(data as CursonQuestionsData).Status = (byte)reader[4];
				(data as CursonQuestionsData).DateTime = (DateTime)reader[5];
				(data as CursonQuestionsData).TotalCount = (data as CursonQuestionsData).ProblemIDS.Length;
				(data as CursonQuestionsData).CorrectCount = (data as CursonQuestionsData).CorrectIDS.Length;
			}
			else if (typeof(T) == typeof(UserQuestionData))
			{
				(data as UserQuestionData).PID = (int)reader[1];
				(data as UserQuestionData).ErrorCount = (byte)reader[2];
				(data as UserQuestionData).CorrectCount = (byte)reader[3];
				(data as UserQuestionData).Status = (byte)reader[4] == 1;
			}
		}
	}

	public class SQLHelper : Singleton<SQLHelper>
	{
		private Dictionary<System.Type, string> TypeMap = new Dictionary<System.Type, string>
		{
			{ typeof(QuestionData), "questions" },
			{ typeof(CursonQuestionsData), "curson_questions" },
			{ typeof(StudentData), "users" },
			{ typeof(UserQuestionData), "user_data" },
		};

		private string config = "server=8.137.125.29;port=3306;user=StudentManager;password=wangxin55;database=studentmanager";
		private MySqlConnection connection = null;
		private MySqlCommand cmd = null;


		public List<T> Query<T>(long? id = null, params string[] queryParams) where T : IDataCommon, new()
		{
			string tableName = string.Empty;
			TypeMap.TryGetValue(typeof(T), out tableName);
			using (var connection = new MySqlConnection(config))
			{
				List<T> result = new List<T>();
				string idName = typeof(T) == typeof(StudentData) ? "user_id" : typeof(T) == typeof(QuestionData) ? "problem_id" : "user_id";

				try
				{
					connection.Open();
					string queryStr = "";
					if (queryParams.Length > 0)
					{
						string fieldList = string.Join(",", queryParams);
						queryStr = $"SELECT {fieldList} FROM {tableName}";

						if (id.HasValue)
							queryStr += $" WHERE {idName} = @id";
					}
					else
					{
						queryStr = $"SELECT * FROM {tableName}";
						if (id.HasValue)
							queryStr += $" WHERE {idName} = @id";
					}

					using (var cmd = new MySqlCommand(queryStr, connection))
					{
						if (id.HasValue)
							cmd.Parameters.AddWithValue("@id", id.Value);

						using (var reader = cmd.ExecuteReader())
						{
							while (reader.Read())
							{
								T row = new T();
								SQLMapping.Mapping(row, reader);
								result.Add(row);
							}
						}
					}

					return result;
				}
				catch (MySqlException ex)
				{
					Console.WriteLine($"MySQL error: {ex.Message}");
					return new List<T> { };
				}
				catch (Exception ex)
				{
					Console.WriteLine($"General error: {ex.Message}");
					return new List<T> { };
				}
			}
		}


		public bool Add<T>(T value) where T : IDataCommon
		{
			string tableName = string.Empty;
			TypeMap.TryGetValue(typeof(T), out tableName);

			using (var connection = new MySqlConnection(config))
			{
				try
				{
					connection.Open();

					if (typeof(T) == typeof(QuestionData))
					{
						var ques = value as QuestionData;
						string sql = $"INSERT INTO {tableName} " +
									 "(problem_id, problem_type, problem_stem, problem_answer, difficulty_level, category, tags, source, lesson, status) " +
									 "VALUES (@Id, @Type, @Stem, @Answer, @DifficultyLevel, @Category, @Tags, @Source, @Lesson, @Status)";

						using (var cmd = new MySqlCommand(sql, connection))
						{
							cmd.Parameters.AddWithValue("@Id", ques.Id);
							cmd.Parameters.AddWithValue("@Type", ques.Type.ToString());
							cmd.Parameters.AddWithValue("@Stem", ques.Stem);
							cmd.Parameters.AddWithValue("@Answer", ques.Answer);
							cmd.Parameters.AddWithValue("@DifficultyLevel", ques.DifficultyLevel.ToString());
							cmd.Parameters.AddWithValue("@Category", ques.Category);
							cmd.Parameters.AddWithValue("@Tags", ques.Tags);
							cmd.Parameters.AddWithValue("@Source", ques.Source);
							cmd.Parameters.AddWithValue("@Lesson", ques.Lesson);
							cmd.Parameters.AddWithValue("@Status", ques.Status.ToString());

							cmd.ExecuteNonQuery();
						}
					}
				}
				catch (MySqlException ex)
				{
					Console.WriteLine($"MySQL error: {ex.Message}");
					return false;
				}
				catch (Exception ex)
				{
					Console.WriteLine($"General error: {ex.Message}");
					return false;
				}
				return true;
			}
		}

		public bool Delete<T>(T value) where T : IDataCommon
		{
			string tableName = string.Empty;
			TypeMap.TryGetValue(typeof(T), out tableName);

			using (var connection = new MySqlConnection(config))
			{
				try
				{
					connection.Open();

					if (typeof(T) == typeof(CursonQuestionsData))
					{
						var ques = value as CursonQuestionsData;
						string sql = "DELETE FROM " +
									 $"{tableName} " +
									 "WHERE user_id = @user_id AND lesson = @lesson";

						using (var cmd = new MySqlCommand(sql, connection))
						{
							cmd.Parameters.AddWithValue("@user_id", ques.UID);
							cmd.Parameters.AddWithValue("@lesson", ques.Lesson);

							cmd.ExecuteNonQuery();
						}
					}
					if (typeof(T) == typeof(QuestionData))
					{
						var ques = value as QuestionData;

						if (ques == null)
						{
							Console.WriteLine($"ERROR: Check Input");
							return false;
						}

						string sql = $"DELETE  " +
									 $"FROM  {tableName}" +
									 "WHERE problem_id = @problem_id";

						using (var cmd = new MySqlCommand(sql, connection))
						{
							cmd.Parameters.AddWithValue("@problem_id", ques.Id);

							cmd.ExecuteNonQuery();
						}

					}
				}
				catch (MySqlException ex)
				{
					Console.WriteLine($"MySQL error: {ex.Message}");
					return false;
				}
				catch (Exception ex)
				{
					Console.WriteLine($"General error: {ex.Message}");
					return false;
				}

				return true;
			}
		}


		internal bool UpdateErrorSet(long UID, ErrorBase error)
		{
			using (MySqlConnection connection = new MySqlConnection(config))
			{
				try
				{
					connection.Open();

					// 检查记录是否存在
					string checkQuery = "SELECT COUNT(*) FROM user_data WHERE user_id = @user_id AND problem_id = @problem_id";
					using (MySqlCommand checkCmd = new MySqlCommand(checkQuery, connection))
					{
						checkCmd.Parameters.AddWithValue("@user_id", UID);
						checkCmd.Parameters.AddWithValue("@problem_id", error.QuestionBase.ID);

						int recordCount = Convert.ToInt32(checkCmd.ExecuteScalar());

						if (recordCount > 0)
						{
							// 记录存在，执行更新操作
							string updateQuery = "UPDATE user_data SET status = @status, error_count = @error_count, correct_count = @correct_count" +
								" WHERE user_id = @user_id AND problem_id = @problem_id";
							using (MySqlCommand updateCmd = new MySqlCommand(updateQuery, connection))
							{
								updateCmd.Parameters.AddWithValue("@user_id", UID);
								updateCmd.Parameters.AddWithValue("@problem_id", error.QuestionBase.ID);
								updateCmd.Parameters.AddWithValue("@error_count", error.ErrorCount);
								updateCmd.Parameters.AddWithValue("@correct_count", error.CorrectCount);
								updateCmd.Parameters.AddWithValue("@status", error.QuestionBase.Status);

								updateCmd.ExecuteNonQuery();
							}
						}
						else
						{
							if (error.QuestionBase.Status == false)
							{
								string updateQuery = $"INSERT INTO user_data (user_id, problem_id) VALUES (@user_id, @problem_id)";
								using (MySqlCommand updateCmd = new MySqlCommand(updateQuery, connection))
								{
									updateCmd.Parameters.AddWithValue("@user_id", UID);
									updateCmd.Parameters.AddWithValue("@problem_id", error.QuestionBase.ID);

									updateCmd.ExecuteNonQuery();
								}
							}
						}
					}
				}
				catch (MySqlException ex)
				{
					Console.WriteLine($"MySQL error: {ex.Message}");
					return false;
				}
				catch (Exception ex)
				{
					Console.WriteLine(ex.Message);
					return false;
				}
				finally
				{
					connection.Close();
				}

				return true;
			}
		}

		internal bool UpdateHomework(long UID, uint lesson, string totalArray, string correctArray, DateTime dateTime, bool status)
		{
			using (MySqlConnection connection = new MySqlConnection(config))
			{
				try
				{
					connection.Open();

					string updateQuery = "UPDATE curson_questions SET status = @status, problem_ids = @problem_ids, " +
										 "correct_ids = @correct_ids, update_time = @update_time WHERE user_id = @user_id AND lesson = @lesson";
					using (MySqlCommand updateCmd = new MySqlCommand(updateQuery, connection))
					{
						updateCmd.Parameters.AddWithValue("@user_id", UID);
						updateCmd.Parameters.AddWithValue("@lesson", lesson);
						updateCmd.Parameters.AddWithValue("@status", status);
						updateCmd.Parameters.AddWithValue("@problem_ids", totalArray);
						updateCmd.Parameters.AddWithValue("@correct_ids", correctArray);
						updateCmd.Parameters.AddWithValue("@update_time", dateTime);

						updateCmd.ExecuteNonQuery();
					}


				}
				catch (MySqlException ex)
				{
					Console.WriteLine($"MySQL error: {ex.Message}");
					return false;
				}
				catch (Exception ex)
				{
					Console.WriteLine(ex.Message);
					return false;
				}
				finally
				{
					connection.Close();
				}
				return true;
			}
		}

		internal bool UpdateErrorSetUserInfo(long UID, uint totalCont, uint totalCorrectCount)
		{
			using (MySqlConnection connection = new MySqlConnection(config))
			{
				try
				{
					connection.Open();

					string updateQuery = "UPDATE users SET errset_totals_questions = @totalCont, errset_correction_count = @totalErrorCount " +
										 " WHERE user_id = @user_id";
					using (MySqlCommand updateCmd = new MySqlCommand(updateQuery, connection))
					{
						updateCmd.Parameters.AddWithValue("@user_id", UID);
						updateCmd.Parameters.AddWithValue("@totalCont", totalCont);
						updateCmd.Parameters.AddWithValue("@totalErrorCount", totalCorrectCount);

						updateCmd.ExecuteNonQuery();
					}


				}
				catch (MySqlException ex)
				{
					Console.WriteLine($"MySQL error: {ex.Message}");
					return false;
				}
				catch (Exception ex)
				{
					Console.WriteLine(ex.Message);
					return false;
				}
				finally
				{
					connection.Close();
				}
				return true;
			}
		}

		internal bool UpdateHomeworkTotalInfo(long UID, uint totalCont, uint totalErrorCount)
		{
			using (MySqlConnection connection = new MySqlConnection(config))
			{
				try
				{
					connection.Open();

					string updateQuery = "UPDATE users SET total_hwq_count = @totalCont, total_hwq_error_count = @totalErrorCount, " +
										 "WHERE user_id = @user_id";
					using (MySqlCommand updateCmd = new MySqlCommand(updateQuery, connection))
					{
						updateCmd.Parameters.AddWithValue("@user_id", UID);
						updateCmd.Parameters.AddWithValue("@totalCont", totalCont);
						updateCmd.Parameters.AddWithValue("@totalErrorCount", totalErrorCount);

						updateCmd.ExecuteNonQuery();
					}


				}
				catch (MySqlException ex)
				{
					Console.WriteLine($"MySQL error: {ex.Message}");
					return false;
				}
				catch (Exception ex)
				{
					Console.WriteLine(ex.Message);
					return false;
				}
				finally
				{
					connection.Close();
				}
				return true;
			}
		}

		internal bool AddHomework(CursonQuestionsData homeWork)
		{
			using (MySqlConnection connection = new MySqlConnection(config))
			{
				try
				{
					connection.Open();

					string updateQuery = "insert into curson_questions (user_id, problem_ids, correct_ids, lesson, status, update_time)" +
						"values (@user_id, @problem_ids, @correct_ids, @lesson, @status, @update_time)";
					using (MySqlCommand updateCmd = new MySqlCommand(updateQuery, connection))
					{
						updateCmd.Parameters.AddWithValue("@user_id", homeWork.UID);
						updateCmd.Parameters.AddWithValue("@problem_ids", JsonConvert.SerializeObject(homeWork.ProblemIDS));
						updateCmd.Parameters.AddWithValue("@correct_ids", JsonConvert.SerializeObject(homeWork.CorrectIDS));
						updateCmd.Parameters.AddWithValue("@lesson", homeWork.Lesson);
						updateCmd.Parameters.AddWithValue("@status", homeWork.Status);
						updateCmd.Parameters.AddWithValue("@update_time", homeWork.DateTime);

						updateCmd.ExecuteNonQuery();
					}

					updateQuery = "UPDATE users SET home_work = @lesson " +
								  "WHERE user_id = @user_id";

					using (MySqlCommand updateCmd = new MySqlCommand(updateQuery, connection))
					{
						updateCmd.Parameters.AddWithValue("@user_id", homeWork.UID);
						updateCmd.Parameters.AddWithValue("@lesson", homeWork.Lesson);

						updateCmd.ExecuteNonQuery();
					}

				}
				catch (MySqlException ex)
				{
					Console.WriteLine($"MySQL error: {ex.Message}");
					return false;
				}
				catch (Exception ex)
				{
					return false;
				}
				finally
				{
					connection.Close();
				}

				return true;
			}
		}

		internal bool UpdateQuestion(QuestionData ques)
		{
			using (MySqlConnection connection = new MySqlConnection(config))
			{
				try
				{
					connection.Open();
					string query = @"UPDATE questions SET 
                                problem_type = @Type, 
                                problem_stem = @Stem, 
                                problem_answer = @Answer, 
                                difficulty_level = @DifficultyLevel, 
                                category = @Category, 
                                tags = @Tags, 
                                source = @Source, 
                                lesson = @Lesson, 
                                status = @Status, 
                                update_time = @UpdateTime 
                             WHERE problem_id = @Id";

					using (MySqlCommand command = new MySqlCommand(query, connection))
					{
						command.Parameters.AddWithValue("@Id", ques.Id);
						command.Parameters.AddWithValue("@Type", ques.Type.ToString());
						command.Parameters.AddWithValue("@Stem", ques.Stem);
						command.Parameters.AddWithValue("@Answer", ques.Answer);
						command.Parameters.AddWithValue("@DifficultyLevel", ques.DifficultyLevel.ToString());
						command.Parameters.AddWithValue("@Category", ques.Category);
						command.Parameters.AddWithValue("@Tags", ques.Tags);
						command.Parameters.AddWithValue("@Source", ques.Source);
						command.Parameters.AddWithValue("@Lesson", ques.Lesson);
						command.Parameters.AddWithValue("@Status", ques.Status.ToString());
						command.Parameters.AddWithValue("@UpdateTime", ques.UpdateTime);

						int rowsAffected = command.ExecuteNonQuery();
						Console.WriteLine($"{rowsAffected} rows updated.");
					}
				}
				catch (MySqlException ex)
				{
					Console.WriteLine($"MySQL error: {ex.Message}");
					return false;
				}
				catch (Exception ex)
				{
					Console.WriteLine(ex.Message);
					return false;
				}
				finally
				{
					connection.Close();
				}
				return true;
			}
		}
	}
}
