﻿using Google.Protobuf.WellKnownTypes;
using Microsoft.VisualBasic;
using StudentManager.Common;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.Json;
using System.Threading.Tasks;
using static Mysqlx.Crud.UpdateOperation.Types;

namespace StudentManager.Data
{
	public class QuestionSetDesc
	{
		public uint TotalQuestions { get; set; } = 0;
		public DateTime UpdateTime { get; set; } = DateTime.Now;
		public uint Lesson { get; set; } = 0;
	}


    public static class QuestionLib
    {
		public static string FileName = "questionLib.dbqi";
		public static string TableDicFileName = "questionTableDicLib.dbqi";
		public static string TableDescFileName = "questionTableDescLib.dbqi";
		public static Dictionary<int, QuestionData> QuestionDic { get; set; } = new Dictionary<int, QuestionData>();
		public static Dictionary<uint, QuestionSetDesc> QuestionTableDesc { get; set; } = new Dictionary<uint, QuestionSetDesc>();
		public static Dictionary<uint, Dictionary<int, QuestionData>> QuestionTableDic { get; set; } = new Dictionary<uint, Dictionary<int, QuestionData>>();
		public static int TotalCount { get { return QuestionDic.Count; } }

		public static void Load()
		{
			QuestionDic.Clear();

			Debug.Assert(Path.Exists(StudentLib.LibPath));

			if (!Path.Exists(StudentLib.LibPath + FileName))
				File.WriteAllText((StudentLib.LibPath + FileName), "");
			string file = File.ReadAllText(StudentLib.LibPath + FileName);
			if (file.Length != 0)
				QuestionDic = JsonSerializer.Deserialize<Dictionary<int, QuestionData>>(file);

			if (!Path.Exists(StudentLib.LibPath + TableDicFileName))
				File.WriteAllText((StudentLib.LibPath + TableDicFileName), "");

			file = File.ReadAllText(StudentLib.LibPath + TableDicFileName);
			if (file.Length != 0)
				QuestionTableDic = JsonSerializer.Deserialize<Dictionary<uint, Dictionary<int, QuestionData>>>(file);

			if (!Path.Exists(StudentLib.LibPath + TableDescFileName))
				File.WriteAllText((StudentLib.LibPath + TableDescFileName), "");
			file = File.ReadAllText(StudentLib.LibPath + TableDescFileName);
			if (file.Length != 0)
				QuestionTableDesc = JsonSerializer.Deserialize<Dictionary<uint, QuestionSetDesc>>(file);
		}
		public static void Save()
		{
			File.WriteAllText((StudentLib.LibPath + FileName), JsonSerializer.Serialize(QuestionDic));
			File.WriteAllText((StudentLib.LibPath + TableDicFileName), JsonSerializer.Serialize(QuestionTableDic));
			File.WriteAllText((StudentLib.LibPath + TableDescFileName), JsonSerializer.Serialize(QuestionTableDesc));
		}

		public static void FreshAllQuestion()
		{

			QuestionDic.Clear ();
			QuestionTableDic.Clear ();
			QuestionTableDesc.Clear ();
			SQLHelper.Instance.Query<QuestionData>().ForEach(x => QuestionDic.Add(x.Id, x));
			SQLHelper.Instance.Query<QuestionData>().ForEach(x => {
				if (!QuestionTableDic.ContainsKey(x.Lesson))
				{
					QuestionTableDic[x.Lesson] = new Dictionary<int, QuestionData>();
				}
				QuestionTableDic[x.Lesson][x.Id] = x;
			});

            foreach (var QuestionTables in QuestionTableDic)
            {
				DateTime update = DateTime.MinValue;
                foreach(var item in QuestionTables.Value)
				{
					update = update > item.Value.UpdateTime ? item.Value.UpdateTime : update;
				}
				QuestionTableDesc.Add(QuestionTables.Key, new QuestionSetDesc{ TotalQuestions = (uint)QuestionTables.Value.Count ,UpdateTime = update, Lesson = QuestionTables.Key });
            }
        }

		public static ObservableCollection<QuestionData> GetAllQuestion()
		{
			var list = new ObservableCollection<QuestionData>();
			foreach (QuestionData data in QuestionDic.Values)
			{
				list.Add(data);
			}
			return list;
		}
		public static ObservableCollection<QuestionData> GetAllQuestionByLesson(uint lesson)
		{
			var list = new ObservableCollection<QuestionData>();
			foreach (QuestionData data in QuestionTableDic[lesson].Values)
			{
				list.Add(data);
			}
			return list;
		}

		public static QuestionData Get(int id)
		{
			if (!QuestionDic.ContainsKey(id)) return null;

			return QuestionDic[id];
		}

		public static QuestionData Get(uint lesson , int id)
		{
			if (!QuestionDic.ContainsKey(id)) return null;

			return QuestionDic[id];
		}

		public static void Add(QuestionData question)
		{
			QuestionDic.Add(question.Id, question);

			if (!QuestionTableDic.ContainsKey(question.Lesson))
			{
				QuestionTableDic[question.Lesson] = new Dictionary<int, QuestionData>();
			}
			QuestionTableDic[question.Lesson][question.Id] = question;

			QuestionTableDesc[question.Lesson].UpdateTime = DateTime.Now;
			QuestionTableDesc[question.Lesson].TotalQuestions = (uint)QuestionTableDic[question.Lesson].Count;

			SQLHelper.Instance.Add(question);
		}

		public static void Remove(QuestionData question)
		{
			if (question == null) throw new NullReferenceException();
			QuestionDic.Remove(question.Id);

			QuestionTableDic[question.Lesson].Remove(question.Id);
			QuestionTableDesc[question.Lesson].TotalQuestions = (uint)QuestionTableDic[question.Lesson].Count;
		}

		public static void Remove(int id)
		{
			QuestionDic.Remove(id);
		}

		internal static void Update(QuestionData selectedQuestion)
		{
			if(QuestionDic[selectedQuestion.Id] != null)
				QuestionDic[selectedQuestion.Id] = selectedQuestion;
			else
				QuestionDic.Add(selectedQuestion.Id, selectedQuestion);
		}
	}
}
